@@ -23,16 +23,16 @@
 #include <string.h>
 #if defined(sun) && defined(__SVR4)
 #include <sys/byteorder.h>
-#else
-#ifdef _WIN32
+#elif defined(_WIN32)
 #define BIG_ENDIAN 'B'
 #define LITTLE_ENDIAN 'l'
 #define BYTE_ORDER LITTLE_ENDIAN
 #define bswap_32(val) ((((val) & 0x000000ffU) << 24) | (((val) & 0x0000ff00U) <<  8) | (((val) & 0x00ff0000U) >>  8) | (((val) & 0xff000000U) >> 24))
-#else
+#elif defined(__linux__)
 #include <byteswap.h>
 #include <endian.h>
-#endif
+#else
+#include <sys/endian.h>
 #endif
 #include <limits.h>
 #include <time.h>
@@ -96,7 +96,11 @@ decode (const void *ptr)
     if ((BYTE_ORDER == BIG_ENDIAN) && sizeof (int) == 4)
         return *(const int *) ptr;
     else if (BYTE_ORDER == LITTLE_ENDIAN && sizeof (int) == 4)
+#ifdef __linux__
         return bswap_32 (*(const int *) ptr);
+#else
+         return bswap32 (*(const uint32_t *) ptr);
+#endif
 #endif
 	else
 	{
@@ -155,18 +159,15 @@ find_transidx (time_t *transitions, ttinfo *types, int *trans_idx, long int num_
 	idx = trans_idx [i];
 	types [idx].isdst ? (*dstidx = i) : (*stdidx = i);
 	
-	if (i >= num_trans - 1) 
-		i--;
-	else 
+	if (i < num_trans - 1) 
 		i++;
+	else 
+		return;
 
 	idx = trans_idx [i];
 	types [idx].isdst ? (*dstidx = i) : (*stdidx = i);
 
-	if (*dstidx != -1 && *stdidx != -1)
-		return;
-	else 
-		icalerror_set_errno (ICAL_MALFORMEDDATA_ERROR);
+	return;
 }
 
 
@@ -198,6 +199,21 @@ icaltzutil_get_zone_directory (void)
 	return zdir;
 }
 
+/* Calculate the relative position of the week in a month from a date */
+static int
+calculate_pos (icaltimetype icaltime)
+{
+	int pos;
+
+	pos = (icaltime.day -1) / 7;
+
+	/* Check if pos 3 is the last occurence of the week day in the month */	
+	if (pos == 3 && ((icaltime.day + 7) > icaltime_days_in_month (icaltime.month, icaltime.year))) 
+		pos = 4;
+
+	return r_pos [pos];
+}
+
 icalcomponent*
 icaltzutil_fetch_timezone (const char *location)
 {
@@ -349,7 +365,7 @@ icaltzutil_fetch_timezone (const char *location)
 			icalrecurrencetype_clear (&ical_recur);
 			ical_recur.freq = ICAL_YEARLY_RECURRENCE;
 			ical_recur.by_month [0] = icaltime.month;
-			pos = r_pos [icaltime.day/7];
+			pos = calculate_pos (icaltime);
 			pos < 0 ? (sign = -1): (sign = 1);
 			ical_recur.by_day [0] = sign * ((abs (pos) * 8) + icaltime_day_of_week (icaltime));
 			icalprop = icalproperty_new_rrule (ical_recur);
@@ -387,7 +403,7 @@ icaltzutil_fetch_timezone (const char *location)
 		icalrecurrencetype_clear (&ical_recur);
 		ical_recur.freq = ICAL_YEARLY_RECURRENCE;
 		ical_recur.by_month [0] = icaltime.month;
-		pos = r_pos [icaltime.day/7];
+		pos = calculate_pos (icaltime);
 		pos < 0 ? (sign = -1): (sign = 1);
 		ical_recur.by_day [0] = sign * ((abs (pos) * 8) + icaltime_day_of_week (icaltime));
 		icalprop = icalproperty_new_rrule (ical_recur);
