@@ -3,7 +3,7 @@
     FILE: icalclassify.c
     CREATOR: ebusboom 23 aug 2000
   
-    $Id: icalclassify.c 596 2007-04-16 06:50:05Z rburton $
+    $Id: icalclassify.c 633 2008-02-18 07:26:45Z pchen $
     $Locker$
     
     (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -267,6 +267,7 @@ int icalssutil_is_rescheduled(icalcomponent* a,icalcomponent* b)
 {
     icalproperty *p1,*p2;
     icalcomponent *i1,*i2;
+    char *temp1, *temp2;
     int i;
 
     icalproperty_kind kind_array[] = {
@@ -285,6 +286,8 @@ int icalssutil_is_rescheduled(icalcomponent* a,icalcomponent* b)
     i2 = icalcomponent_get_first_real_component(b);
 
     for(i =0; kind_array[i] != ICAL_NO_PROPERTY; i++){
+	int cmp;
+
 	p1 = icalcomponent_get_first_property(i1,kind_array[i]);
 	p2 = icalcomponent_get_first_property(i2,kind_array[i]);
 	
@@ -292,10 +295,16 @@ int icalssutil_is_rescheduled(icalcomponent* a,icalcomponent* b)
 	    /* Return true if the property exists in one component and not
 	       the other */
 	    return 1;
-	}
+	} else if (!p1 && !p2) 
+		continue;
+
+	temp1 = icalproperty_as_ical_string (p1);
+	temp2 = icalproperty_as_ical_string (p2);
 	
-	if(p1 && strcmp(icalproperty_as_ical_string(p1),
-			icalproperty_as_ical_string(p2)) != 0){
+	cmp = strcmp (temp1, temp2);
+	free (temp1);
+	free (temp2);
+	if(p1 &&  cmp  != 0){
 	    return 1;
 	}
     }
