@@ -372,7 +372,9 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 		case CAMEL_FOLDER_ARG_VISIBLE:
 			/* This is so we can get the values atomically, and also so we can calculate them only once */
 
-			#warning "Add a better base class function to get counts specific to normal/vee folder."
+			/* FIXME[disk-summary] Add a better base class
+			 * function to get counts specific to normal/vee
+			 * folder. */
 			if (unread == -1) {
 
 				if (1) {
@@ -402,7 +404,9 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 						}
 
 					}*/
-                                        #warning "I added it for vfolders summary storage, does it harm ?"
+                                        /* FIXME[disk-summary] I added it for
+					 * vfolders summary storage, does it
+					 * harm ? */
 					if (unread == -1) {
 						//unread = folder->summary->unread_count;
 						/*
@@ -1139,24 +1143,9 @@ get_uids(CamelFolder *folder)
 	GPtrArray *array;
 	int i, j, count;
 
-	array = g_ptr_array_new();
+	g_return_val_if_fail(folder->summary != NULL, g_ptr_array_new ());
 
-	g_return_val_if_fail(folder->summary != NULL, array);
-
-	count = camel_folder_summary_count(folder->summary);
-	g_ptr_array_set_size(array, count);
-	for (i = 0, j = 0; i < count; i++) {
-		CamelMessageInfo *info = camel_folder_summary_index(folder->summary, i);
-		
-		if (info) {
-			array->pdata[j++] = (char *)camel_pstring_strdup (camel_message_info_uid (info));
-			camel_message_info_free(info);
-		}
-	}
-	
-	g_ptr_array_set_size (array, j);
-	
-	return array;
+	return camel_folder_summary_array (folder->summary);
 }
 
 
@@ -1366,10 +1355,11 @@ count_by_expression (CamelFolder *folder, const char *expression,
 		      CamelException *ex)
 {
 	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-			      "");
+			      _("Unsupported operation: count by expression: for %s"),
+			      camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder)));
 	
-g_warning ("CamelFolder::count_by_expression not implemented for "
-		     "'%s'", camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder)));
+	w(g_warning ("CamelFolder::count_by_expression not implemented for "
+		     "'%s'", camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder))));
 	
 	return 0;
 }
