@@ -5,7 +5,7 @@
  *          Michael Zucchi <notzed@ximian.com>
  *          Jeffrey Stedfast <fejj@ximian.com>
  *
- * Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of version 2 of the GNU Lesser General Public 
@@ -32,8 +32,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <libedataserver/e-iconv.h>
-
 #include "camel-charset-map.h"
 #include "camel-html-parser.h"
 #include "camel-mime-filter-basic.h"
@@ -61,18 +59,18 @@ simple_data_wrapper_construct_from_parser (CamelDataWrapper *dw, CamelMimeParser
 	GByteArray *buffer;
 	CamelStream *mem;
 	size_t len;
-	
+
 	d(printf ("simple_data_wrapper_construct_from_parser()\n"));
-	
+
 	/* read in the entire content */
 	buffer = g_byte_array_new ();
 	while (camel_mime_parser_step (mp, &buf, &len) != CAMEL_MIME_PARSER_STATE_BODY_END) {
 		d(printf("appending o/p data: %d: %.*s\n", len, len, buf));
-		g_byte_array_append (buffer, buf, len);
+		g_byte_array_append (buffer, (guint8 *) buf, len);
 	}
-	
+
 	d(printf("message part kept in memory!\n"));
-	
+
 	mem = camel_stream_mem_new_with_byte_array (buffer);
 	camel_data_wrapper_construct_from_stream (dw, mem);
 	camel_object_unref (mem);
