@@ -32,7 +32,6 @@
 
 #include <camel/camel-object.h>
 #include <camel/camel-service.h>
-#include <camel/camel-db.h>
 
 G_BEGIN_DECLS
 
@@ -117,15 +116,21 @@ typedef struct _CamelRenameInfo {
 #define CAMEL_STORE_VJUNK		(1 << 3)
 #define CAMEL_STORE_PROXY		(1 << 4)
 
+struct _CamelDB;
+
 struct _CamelStore {
 	CamelService parent_object;
 	struct _CamelStorePrivate *priv;
 	
 	CamelObjectBag *folders;
-	CamelDB *cdb;
+	struct _CamelDB *cdb_r;
+	struct _CamelDB *cdb_w;
 
 	guint32 flags;
 	guint32 mode;
+
+	/* Future ABI expansion */
+	gpointer later[4];
 };
 
 /* open mode for folder */
@@ -136,7 +141,9 @@ struct _CamelStore {
 
 #define CAMEL_STORE_FOLDER_CREATE_EXCL (CAMEL_STORE_FOLDER_CREATE | CAMEL_STORE_FOLDER_EXCL)
 
+#ifndef CAMEL_DISABLE_DEPRECATED
 #define CAMEL_STORE_FOLDER_INFO_FAST       (1 << 0)
+#endif /* CAMEL_DISABLE_DEPRECATED */
 #define CAMEL_STORE_FOLDER_INFO_RECURSIVE  (1 << 1)
 #define CAMEL_STORE_FOLDER_INFO_SUBSCRIBED (1 << 2)
 #define CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL (1 << 3)  /* don't include vTrash/vJunk folders */
@@ -217,7 +224,7 @@ void             camel_store_delete_folder      (CamelStore *store,
 						 const char *folder_name,
 						 CamelException *ex);
 void             camel_store_rename_folder      (CamelStore *store,
-						 const char *old_name,
+						 const char *old_namein,
 						 const char *new_name,
 						 CamelException *ex);
 
@@ -237,10 +244,12 @@ void             camel_store_free_folder_info_nop  (CamelStore *store,
 
 CamelFolderInfo *camel_folder_info_new             (void);
 void             camel_folder_info_free            (CamelFolderInfo *fi);
+#ifndef CAMEL_DISABLE_DEPRECATED
 CamelFolderInfo *camel_folder_info_build           (GPtrArray *folders,
 						    const char *namespace,
 						    char separator,
 						    gboolean short_names);
+#endif /* CAMEL_DISABLE_DEPRECATED */
 CamelFolderInfo *camel_folder_info_clone	   (CamelFolderInfo *fi);
 
 gboolean         camel_store_supports_subscriptions   (CamelStore *store);
@@ -265,6 +274,7 @@ gboolean         camel_store_can_refresh_folder       (CamelStore *store,
 						       CamelFolderInfo *info,
 						       CamelException *ex);
 
+#ifndef CAMEL_DISABLE_DEPRECATED
 typedef struct _CamelISubscribe CamelISubscribe;
 struct _CamelISubscribe {
 	CamelInterface iface;
@@ -278,6 +288,7 @@ CamelType camel_isubscribe_get_type (void);
 gboolean camel_isubscribe_subscribed(CamelStore *store, const char *name);
 void camel_isubscribe_subscribe(CamelStore *store, const char *folder_name, CamelException *ex);
 void camel_isubscribe_unsubscribe(CamelStore *store, const char *folder_name, CamelException *ex);
+#endif /* CAMEL_DISABLE_DEPRECATED */
 
 G_END_DECLS
 
